// hlavickove subory *******************************************************
// systemove funkcie
#include <stdio.h>
#include <stdlib.h>

// .h subor
#include "BMPImage.h"

// konstruktor *************************************************************
CBMPImage::CBMPImage()
{

}

// destruktor **************************************************************
CBMPImage::~CBMPImage()
{

}

// nacitanie suboru s BMP obrazkom *****************************************
BMPIMAGE *CBMPImage::open_bitmap(char *fname)
{
 BMPIMAGE *bmp;

 bmp=new BMPIMAGE;
 if (bmp==NULL) {
  printf("Error allocating memory for BMPIMAGE: %s\n",fname);
  return NULL;
 }

 FILE *file;
 unsigned long size;        // velkost obrazka v bytoch
 unsigned long i;
 unsigned short int planes; // pocet planes v obrazku (potrebne 1)
 unsigned short int bpp;    // pocet bitov na pixel (potrebne 24)
 unsigned char temp;        // pomocna premenna pre bgr-rgb konverziu

 // otvorenie suboru
 if ((file=fopen(fname,"rb"))==NULL) { // 'b' je v linuxe ignorovane
  printf("File not found: %s\n",fname);
  return NULL;
 }
 // INFO: meno otvoreneho suboru
 //printf("meno obrazka: %s\n",fname);

 // posun v BMP hlavicke na udaje o sirke a vyske obrazka
 fseek(file,18,SEEK_CUR);

 // nacitanie sirky
 if ((i=fread(&bmp->w,4,1,file))!=1) { // 4 byty
  printf("Error reading width from: %s\n",fname);
  return NULL;
 }
 // INFO: sirka obrazka
 //printf("sirka obrazka %s: %u\n",fname,bmp->w);

 // nacitanie vysky
 if ((i=fread(&bmp->h,4,1,file))!=1) {
  printf("Error reading height from: %s\n",fname);
  return NULL;
 }
 // INFO: vyska obrazka
 //printf("vyska obrazka %s: %u\n",fname,bmp->h);

 // vypocet velkosti obrazka (3 byty na pixel)
 size=bmp->w*bmp->h*3*sizeof(unsigned char);

 // nacitanie planes
 if ((fread(&planes,2,1,file))!=1) {
  printf("Error reading planes from: %s\n",fname);
  return NULL;
 }
 if (planes!=1) {
  printf("Planes from: %s is not 1: %u\n",fname,planes);
  return NULL;
 }

 // nacitanie bpp
 if ((i=fread(&bpp,2,1,file))!=1) {
  printf("Error reading bpp from: %s\n",fname);
  return NULL;
 }
 if (bpp!=24) {
  printf("Bpp from: %s is not 24: %u\n",fname,bpp);
  return NULL;
 }

 // posun na koniec BMP hlavicky
 fseek(file,24,SEEK_CUR);

 // nacitanie samotnych udajov
 bmp->data=(unsigned char *) malloc(size);
 if (bmp->data==NULL) {
  printf("Error allocating memory for color-corrected image data");
  return NULL;
 }
 // INFO: pocet alokovanych bytov
 //printf("%lu alokovanych bytov\n",size);

 if ((i=fread(bmp->data,size,1,file))!=1) {
  printf("Error reading image data from: %s\n",fname);
  return NULL;
 }

 for (i=0;i<size;i+=3) { // korekcia nacitanych farieb (bgr->rgb)
  temp=bmp->data[i];
  bmp->data[i]=bmp->data[i+2];
  bmp->data[i+2]=temp;
 }

 fclose(file);

 return bmp;
}

// nastavenie priehladnosti ************************************************
void CBMPImage::set_bitmap_transparent(BMPIMAGE *bmp,unsigned char r,
                                                     unsigned char g,
                                                     unsigned char b)
{
 unsigned int x,y,i=0;

 for (y=0;y<bmp->h;y++) {
  for (x=0;x<bmp->w;x++) {
   if ((bmp->data[i+0]==b)&&(bmp->data[i+1]==g)&&(bmp->data[i+2]==r))
    bmp->data[i+3]=0;
   i+=4;
  }
 }
}

// uvolnenie pouzitej pamate ***********************************************
void CBMPImage::close_bitmap(BMPIMAGE *bmp)
{
 if (bmp!=NULL) {
  if (bmp->data!=NULL) free(bmp->data);
  bmp->data=NULL;
  delete bmp;
  bmp=NULL;
 }
}

